# Example fully crossed
# Experimental!

# -- Simulate --
N_pers=30; # Number of persons
sd_pers=5; # Standard deviation of latent variable for persons
N_levels=30; # Number of Levels 
sd_levels=5; # Standard deviation of latent variable for Levels
sd_error=0.5; # Residual variance for all items.

# Generate "true" factor values for persons
pers_factor=rnorm(N_pers, sd=sd_pers)
# Generate "true" factor values for levels
level_factor=rnorm(N_levels, sd=sd_levels)

# Initialize objects 
subj<-c()
lev<-c()
y1<-c()
y2<-c()
y3<-c()

# Construct linear predictors for observed
# variables based on latent variables
counter=1;
for(i in 1:N_pers)
{
  for(j in 1:N_levels)
  {
    subj[counter]<-i
    lev[counter]<-j
    y1[counter]<-1*pers_factor[i]+1.5*level_factor[j]
    y2[counter]<-2*pers_factor[i]+2.5*level_factor[j]
    y3[counter]<-pers_factor[i]+level_factor[j]
    counter=counter+1;
  }
}

# Add errof variance
y1<-y1+rnorm(length(y1), sd=sd_error)
y2<-y2+rnorm(length(y2), sd=sd_error)
y3<-y3+rnorm(length(y3), sd=sd_error)

# Join variables into a data frame
frame=data.frame(subj, lev, y1, y2, y3)

# Dump the data to disk for external crunching
write.table(frame, file="cc.dat",  col.names=FALSE, row.names=FALSE)

# ---- Estimate ----
# Load the required libraries
library(R2jags)
library(mcmcplots)

n=length(y1); # Number of observations

dat<-list("y1","y2", "y3","subj", "N_pers", "lev", "N_levels","n")
parameters<-c("v.eta.s", "v.eta.l", "v1","v2","v3", "lambda1", "lambda2", "gamma1", "gamma2")

# Estimate with JAGs using multiple cores
output<-jags.parallel(dat, inits=NULL, parameters,  model.file="test.txt",n.chains=2, n.iter=10000, n.burnin=9000)
attach.jags(output);

# ---- Check the results ----
# Print the results
print(output);
# Inspect mixing
traplot(output);
 # Inspect densities
denplot(output);
# Convert to mcmc object
out_mcmc<-as.mcmc(output)
# Check autocorrelation for gamma2
autplot1(out_mcmc[,"gamma2", drop=F], lag.max=10000)